(ns wanijo.schema.view
  (:require [clojure.string :refer [capitalize]]
            [hiccup.form :as hform]
            [ring.util.anti-forgery :refer [anti-forgery-field]]
            [formulare.core :as form]
            [wanijo.framework.view :as view]
            [wanijo.framework.routing :refer [path]]
            [wanijo.framework.time :refer [prettify-dt]]
            [wanijo.schema.domain :as domain]
            [wanijo.attribute.domain :as attr-domain]))

(def form
  {:fields {:name {:label "Name"
                   :required true
                   :spec ::domain/name}
            :uuid {:widget :hidden}}})

(def attr-form
  {:fields {:name {:label "Name"
                   :required true
                   :spec ::attr-domain/name}
            :type {:label "Type"
                   :required true
                   :spec ::attr-domain/type
                   :options (map #(vector (capitalize %) %)
                                 attr-domain/types)}
            :required {:label "Required"
                       :required false
                       :spec ::attr-domain/required
                       :widget :checkbox}
            :uuid {:widget :hidden}}})

(def assign-form
  {:fields {:assigned {:label "Users"
                       :required false
                       :spec ::domain/assigned-to
                       :widget :mselect
                       :from-req #(if (vector? %) % [%])}
            :uuid {:widget :hidden}}})

(def schema-connections-form
  {:fields {:connections {:label "Schemas"
                          :required false
                          :spec ::domain/assigned-to
                          :widget :mselect
                          :from-req #(if (vector? %) % [%])}
            :uuid {:widget :hidden}}})

(defn overview! [req]
  (let [session (:session req)
        uuid (:uuid session)
        schemas (domain/all-created-by! uuid)]
    (view/layout!
     :session session
     :content
     [[:h1 "All schemas"]
      [:table
       [:thead
        [:tr
         [:th "Name"]
         [:th "Created"]]]
       [:tbody
        (for [schema schemas]
          [:tr
           [:td
            [:a {:href (path :schema-show schema)}
             (:name schema)]]
           [:td
            (prettify-dt (:created_at schema))]])]]
      [:h1 "New schema"]
      (hform/form-to [:post (path :schema-new)]
                     (form/render-widgets form {} req)
                     (hform/submit-button "Create"))])))

(defn show-schema! [schema attrs assign-form conn-form req]
  (view/layout!
   :request req
   :content
   [[:h1 "Schema "
     [:span.schema-title__name (:name schema)]]
    [:h2 "Edit"]
    (hform/form-to [:post (path :schema-edit)]
                   (form/render-widgets form schema req)
                   (hform/submit-button "Edit"))
    [:h2 "Permissions"]
    [:h3 "Read permissions"]
    (hform/form-to [:post (path :schema-assign-users)]
                   (form/render-widgets assign-form
                                        (assoc schema :assigned
                                               (:assigned-read-users schema))
                                        req)
                   (hform/hidden-field "permission" "read")
                   (hform/submit-button "Assign"))
    [:h3 "Write permissions"]
    (hform/form-to [:post (path :schema-assign-users)]
                   (form/render-widgets assign-form
                                        (assoc schema :assigned
                                               (:assigned-write-users schema))
                                        req)
                   (hform/hidden-field "permission" "write")
                   (hform/submit-button "Assign"))
    [:h3 "Allowed schema connections"]
    (hform/form-to [:post (path :schema-assign-schemas)]
                   (form/render-widgets conn-form
                                        (assoc schema
                                               :connections
                                               (:assigned-schemas schema))
                                        req)
                   (hform/submit-button "Assign"))
    [:h2 "Attributes"]
    [:ul.schema-attributes
     (for [attr attrs]
       [:li
        (hform/form-to [:post (path :attribute-edit {:schema (:uuid schema)})]
                       (form/render-widgets attr-form attr req)
                       (hform/submit-button "Save"))
        (hform/form-to [:delete (path :attribute-delete attr)]
                       (anti-forgery-field)
                       (hform/hidden-field "schema" (:uuid schema))
                       (view/delete-btn)
                       [:label "Created at "
                        [:em (prettify-dt (:created_at attr))]])])]
    [:h3 "New attribute"]
    (hform/form-to [:post (path :attribute-new)]
                   (form/render-widgets attr-form {} req)
                   (hform/hidden-field "schema" (:uuid schema))
                   (hform/submit-button "Create"))
    [:h2 "Actions"]
    (hform/form-to {:class "inline"}
                   [:delete (path :schema-delete schema)]
                   (anti-forgery-field)
                   (view/delete-btn))]))
